/*
# Copyright (c) 2003-2004 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Purushotham Kamath
#                 Sept  2004
#This material is based upon work supported by the Defense Advanced Research 
#Projects Agency under contract no. N66001-02-1-8939 issued by the Space and 
#Naval Warfare Systems Center (SPAWAR).  Any opinions, findings, and conclusions 
#or recommendations expressed in this material are those of the author(s) and 
#do not necessarily reflect the views of the Defense Advanced Research Projects
#Agency, SPAWAR, or the U.S. Government.
*/


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <defs.h>
#include <debug.h>
#include <packet.h>
#include <eventhandler.h>
#include <queue.h>
#include <codeset.h>

//The codeset class is used to store the codeset.
//The codeset is stored in a linear array, indexed by the 
//length of the codeword, the number of wavelengths

//The class contains functions to intialize the codeset 
//from a file and to shift (rotate) individual codewords



Codeset::Codeset(int n, int a, int W, int k)
{
	N = n;
	w = W;
	K = k;
	A =a;
	InitializeCodeset(N, A, w, K);
}



void Codeset::InitializeCodeset(int N, int A, int w, int K)
{
	char codefile[40];
	sprintf(codefile, "codes.%d.%d.%d.%d.txt", N, A, w, K);
	FILE *cf = fopen(codefile, "r");
        if (cf == NULL)
        {
                printf("TrGen: Error opening file: %s\n", codefile);
                exit(-1);
        }

        int error = fscanf(cf, "%d", &numcodewords);
        if(error < 0)
        {
                printf("TrGen: Error reading file\n");
                exit(-1);
        }
        dprintf("Num code: %d\n", numcodewords);

        int weight;
        error = fscanf(cf, "%d", &weight);
        if(error < 0)
        {
                printf("TrGen: Error reading file\n");
                exit(-1);
        }
        dprintf("Weight: %d\n", weight);

        set = new int [numcodewords * A * N];
        for(int count = 0; count < numcodewords; count++)
        {
		for (int count2 =0; count2 < A; count2++)	
		{
			int chip;	
                	dprintf("Codeset::IntializeCodeset: Index: %d\n", count);
			for (int count3 =N-1; count3 >=0; count3--)	
			{
				//Convention: MSB is read first
				fscanf(cf, "%d", &chip);
				set[count * (A*N) + count2 * N + count3] = chip;
			}
		}
		DumpCodeword(count);

        }
        fclose(cf);

}

int * Codeset:: RightShiftCodeword(int codeword, int delay)
{
	//printf("Codeword: %d\n", codeword);
	//printf("Delay: %d\n", delay);

	int * newcodeword = new int[A*N];
	for (int count2 =0; count2 < A; count2++)	
	{	
		for (int count3 =0; count3 < N; count3++)	
		{
		 	int shift = (count3 + delay) % N;
			//printf("Shift: %d\n", shift);
			newcodeword[count2*(N) + count3] = set[codeword*(A*N) + count2*N +  shift];
		}
	}
	return newcodeword;
	
}
int * Codeset:: LeftShiftCodeword(int codeword, int delay)
{
	//printf("Codeword: %d\n", codeword);
	//printf("Delay: %d\n", delay);

	int * newcodeword = new int[A*N];
	for (int count2 =0; count2 < A; count2++)	
	{	
		for (int count3 =0; count3 < N; count3++)	
		{
		 	int shift = (count3 + N -delay) % N;
			//printf("Shift: %d\n", shift);
			newcodeword[count2*(N) + count3] = set[codeword*(A*N) + count2*N + shift];
	
		}
	}
	return newcodeword;
}	

int * Codeset:: RightShiftCodeword(int *codeword, int delay)
{
	//printf("Codeword: %d\n", codeword);
	//printf("Delay: %d\n", delay);

	int * newcodeword = new int[A*N];
	for (int count2 =0; count2 < A; count2++)	
	{	
		for (int count3 =0; count3 < N; count3++)	
		{
		 	int shift = (count3 + delay) % N;
			//printf("Shift: %d\n", shift);
			newcodeword[count2*(N) + count3] = codeword[count2*N + shift];
		}
	}
	return newcodeword;
	
}
int * Codeset:: LeftShiftCodeword(int *codeword, int delay)
{
	//printf("Codeword: %d\n", codeword);
	//printf("Delay: %d\n", delay);

	int * newcodeword = new int[A*N];
	for (int count2 =0; count2 < A; count2++)	
	{	
		for (int count3 =0; count3 < N; count3++)	
		{
		 	int shift = (count3 + N - delay) % N;
			//printf("Shift: %d\n", shift);
			newcodeword[count2*(N) + count3] = codeword[count2*N +  shift];
	
		}
	}
	return newcodeword;
}	

void Codeset:: DumpCodeword(int index)
{
	for (int count2 =0; count2 < A; count2++)	
	{	
		for (int count3 =N-1; count3 >=0; count3--)	
		{

			::dprintf("%d ", set[index * (A*N) + count2 * N + count3]);
		}
		::dprintf("\n");
	}
}


