/*
# Copyright (c) 2003-2004 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Purushotham Kamath
#                 Sept  2004
#This material is based upon work supported by the Defense Advanced Research 
#Projects Agency under contract no. N66001-02-1-8939 issued by the Space and 
#Naval Warfare Systems Center (SPAWAR).  Any opinions, findings, and conclusions 
#or recommendations expressed in this material are those of the author(s) and 
#do not necessarily reflect the views of the Defense Advanced Research Projects
#Agency, SPAWAR, or the U.S. Government.
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <defs.h>
#include <packet.h>
#include <eventhandler.h>
#include <queue.h>
#include <squeue.h>
#include <scheduler.h>
#include <state.h>
#include <rng.h>
#include <trgen.h>
#include <trdump.h>
#include <receiver.h>
#include <intransit.h>
#include <state.h>
#include <codeset.h>
#include <txsched.h>
#include <stateest.h>

#include <debug.h>

Scheduler *sched;
TrGen *trgen;
TrDump *trdump;
InTransit *it;
Codeset *codeset;
Receiver *rcvr;
TxSched *txsched;
StateEstimator *stateest;

int * nstate;

char inputfile[20];
char outputfile[20];

int propSpeed = (int)2E8;

int debug = 0;

int main(int argc,  char **argv)
{

	
	if(argc != 16)
	{
		printf("Usage: simone <seed> <N> <A> <w> <K> <Chip rate in c/s> <Traffic type> <Input traffic file> <Output traffic file> <Average packet size in bytes> <Normalized offered load> <Number of packets> <Average distance of node in m> <Tx scheduling algorithm> <Threshold>\n");
		printf("<Tx scheduling algorithm> 0-Aloha 1-Selfish 2-Threshold 3-Overlap section 4-Overlap size\n");
		printf("<Traffic> 0-Poisson arrivals/Exponential packet lengths 1-Uniform/Uniform 2-Deterministic/Deterministic 3-Poisson/Trimodal 4-From traffic file\n");
		exit(0);
	}

	int seed = atoi(argv[1]);
	srandom(seed);
	
	int N = atoi(argv[2]);
	int A = atoi(argv[3]);
	int w = atoi(argv[4]);
	int K = atoi(argv[5]);

	int traffic = atoi(argv[7]);
	strcpy(inputfile, argv[8]);
	strcpy(outputfile, argv[9]);

	dprintf("main.cc: main(): Arguments: Seed: %d\n", seed);
	dprintf("main.cc: main(): Arguments: Input traffic file: %s\n", inputfile);

	sched = new Scheduler();
	codeset = new Codeset(N, A, w, K);
	codeset->chiprate = atof(argv[6]);
	dprintf("main.cc: main(): Arguments: Chip rate: %.6f\n", codeset->chiprate);
	it = new InTransit(A, N);

	trgen = new TrGen(traffic);
	if(traffic == TRAFFIC_MTM)
		trgen->avgpktsize = 0.7*40 + 0.2*1500 + 0.10*576;
	else
		trgen->avgpktsize = atof(argv[10]);


	double offeredload = atof(argv[11]);
	double avginterarrival = (1.0/offeredload) * (double)trgen->avgpktsize *8 /(codeset->chiprate * A);
	trgen->avginterarrival = avginterarrival; 
	trgen->totalpkts = atoi(argv[12]);

	trgen->avgdistance = atof(argv[13]);

	int alg = atoi(argv[14]);
	double th = atof(argv[15]);
	switch (alg)
	{
		case TXSCHED_ALOHA:	
			txsched = new TxSched_Aloha();
			break;
		case TXSCHED_SELFISH:
			txsched = new TxSched_Selfish(N, A, w, K);
			break;
		case TXSCHED_THRESHOLD:
			txsched = new TxSched_Threshold(N, A, w, K, th);
			break;
		case TXSCHED_OVERLAPSECTION:
			txsched = new TxSched_OverlapSection(N, A, w, K, th);
			break;
		case TXSCHED_OVERLAPSIZE:
			txsched = new TxSched_OverlapSize(N, A, w, K, th);
			break;
		case TXSCHED_OPTIMALSTATE:
			txsched = new TxSched_OptimalState(N, A, w, K);
			break;
//		case TXSCHED_NUMUSERS:
//			txsched = new TxSched_NumUsers(N, A, w, K);
//			break;
	}

	stateest = new StateEstimator( N, A);		
	trdump = new TrDump();
	trdump->offeredload = offeredload;
	dprintf("main.cc: main(): Initialization complete. Reading traffic\n");
	trgen->read();

	rcvr = new CorrelationReceiver(N, A, w, K);


//nstate is a normalized state variable used to validate the state estimator calculations
//It does not have any impact on the working of the simulator and may be removed without
//affecting operation.
	nstate = new int[A*N];

	while (1)
		if(!(sched->run()))
			break;

	dprintf("main.cc: main(): No more events to schedule\n");

	trdump->dumpstats();
	stateest->dumpstats();
	delete trdump;
	delete it;
	delete trgen;
	delete sched;
	exit(0);
}
