/*
# Copyright (c) 2003-2004 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Purushotham Kamath
#                 Sept  2004
#This material is based upon work supported by the Defense Advanced Research 
#Projects Agency under contract no. N66001-02-1-8939 issued by the Space and 
#Naval Warfare Systems Center (SPAWAR).  Any opinions, findings, and conclusions 
#or recommendations expressed in this material are those of the author(s) and 
#do not necessarily reflect the views of the Defense Advanced Research Projects
#Agency, SPAWAR, or the U.S. Government.
*/


#ifndef PACKET_H
#define PACKET_H

//Abstraction of a packet.
//Contains some fields usually present in link layer, network layer and transport layer headers
//Contains physical layer transmission parameters (Codeset paramters)
//Also contains fields needed for simulation (arrival time of packet, time of departure, location of departure)
//Also contains statistics (all layers)
class PacketState;
 
class Packet 
{
private: 
public:

//List data members
	Packet *next;
	Packet *prev;

//Packet header
	int id;      //Packet identification.

//Link/Network/Transport layer fields 
	int srcaddr; //Source address.
	int srcport; //Source port.
	int dstaddr; //Destination address.
	int dstport; //Destination port.
	int size;    //Total size of packets (all headers + data) in bytes.

//Physical layer transmsision parameters
	double bitrate;
	double chiprate;
	
	int codeword;		//index into codeset array
	int A;
	int w;
	int N;

//Departure point (Point in space where transmission of the packet began).
	double depx;
	double depy;
	double depz;

//Arrival time (Point in time when the packet arrived at the interfeace of the node for transmission).
	double arrivaltime;
//Departure time (Point in time when the transmission of the packet (first bit) began).
	double departuretime;
	int offset;

	int status;

	int backoffs;

	Packet();
	Packet(Packet &);
	void calcboundary(double *, double *);
	int *calcstate(double x, double y, double z, double time, int topology );
	void calcnormalizedstate();
	virtual ~Packet();
	int val(int, int);

protected:


};

enum{PACKET_OK, PACKET_ERROR_MUI};



#endif

