/*
# Copyright (c) 2003-2004 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Purushotham Kamath
#                 Sept  2004
#This material is based upon work supported by the Defense Advanced Research 
#Projects Agency under contract no. N66001-02-1-8939 issued by the Space and 
#Naval Warfare Systems Center (SPAWAR).  Any opinions, findings, and conclusions 
#or recommendations expressed in this material are those of the author(s) and 
#do not necessarily reflect the views of the Defense Advanced Research Projects
#Agency, SPAWAR, or the U.S. Government.
*/

#include <stdlib.h>
#include <stdarg.h>

#include <defs.h>
#include <packet.h>
#include <eventhandler.h>
#include <squeue.h>
#include <debug.h>

//SQueue class functions.

SQueue::SQueue(int s): size(s *1024), bufferoverflows(0), byteoverflows(0)
{
	qsize =0;
	avgqsize =0;
	maxqsize =0;
	pktcnt =0;
	head = NULL;
	tail = NULL;	
	bytes =0;
}

SQueue::~SQueue()
{

}

int SQueue:: enqueue(Packet *pkt, Packet *followingpkt)
{

	dprintf("SQueue.cc: SQueue::enSQueue(): Entered function\n");	


	if (followingpkt == NULL)
	{

//Kamath: 07/09/01: Bug fix
//Check the SQueue size limitation for the case when there are no packets in 
//the SQueue
		if(pkt->size > size)
		{
			dprintf("SQueue.cc: SQueue::enSQueue(): Buffer overflow.\n");	
			bufferoverflows++;
			byteoverflows+=pkt->size;
			return 0;
		}

		qsize++;
		bytes+=pkt->size;


		if(qsize > maxqsize)
			maxqsize=qsize;	


		if(head == NULL)
		{

			head = pkt;
			tail = pkt;
			pkt->next = NULL;
			pkt->prev = NULL;
		}
		else
		{
			tail->next = pkt;
			pkt->next = NULL;
			pkt->prev = tail;
			tail =pkt;
		}

		return 1;
	}


//If supposed to add before head
	if(followingpkt == head)
	{

		pkt->prev = NULL;
		pkt->next = head;
		head->prev = pkt;
		head= pkt;
		return 0;
	}


//If supposed to add somewhere in the queue
	Packet *p = head;
	while(p->next != NULL)
	{
	
		if(p->next == followingpkt)
		{

			pkt->prev = p;
			pkt->next = p->next;
			p->next = pkt;
			pkt->next->prev = pkt;
			break;
		}

		p = p->next;

	}
			
	return 0;
}

Packet * SQueue:: dequeue()
{
	dprintf("SQueue.cc: SQueue::deSQueue(): Entered function\n");	
	if(head != NULL)
	{
		qsize--;
		Packet *p = head;
		bytes-=p->size;
		head = head->next;
		if (head == NULL)
			tail = NULL;
		dprintf("SQueue.cc: SQueue::deSQueue(): Leaving function\n");	
		return p;
	}
	else 
	{
		dprintf("SQueue.cc: SQueue::deSQueue(): Nothing in SQueue: Leaving function\n");	
		return NULL;
	}
}

Packet * SQueue:: dequeue(Packet *rp)
{
	dprintf("SQueue.cc: SQueue::deSQueue2(): Entered function\n");	
	if(head != NULL)
	{
		qsize--;
		Packet *p = head;
		while(p != rp)
		{
			if(p  ==NULL)
				return NULL;
			p=p->next;
		}
	
		if (p== NULL)
		{
			dprintf("SQueue.cc: SQueue::deSQueue2(): Not found\n");	
			return NULL;
		}
		
		bytes-=p->size;

		dprintf("SQueue.cc: SQueue::deSQueue2(): Head\n");	
		if(p == head)
		{
			head = p->next;
			if (tail == p)
				tail = p->next;
			return p;
		}

		dprintf("SQueue.cc: SQueue::deSQueue2(): Tail\n");	
		if(p == tail)
		{
			tail = p->prev;
			tail->next  = NULL;
			return p;
		}

		dprintf("SQueue.cc: SQueue::deSQueue2(): Neither\n");	
		p->prev->next  = p->next;
		p->next->prev = p->prev;

		dprintf("SQueue.cc: SQueue::deSQueue(): Leaving function\n");	
		return p;
	}
	else 
	{
		dprintf("SQueue.cc: SQueue::deSQueue(): Nothing in SQueue: Leaving function\n");	
		return NULL;
	}
}




void SQueue::dumpstats()
{
	printf("SQueue.cc: SQueue::dumpstats(): Number of bufferoverflows: %d\n", bufferoverflows);
	printf("SQueue.cc: SQueue::dumpstats(): Byte overflows: %d\n", byteoverflows);
}
